<?php

namespace Flow\Import\LiquidThreadsApi;

use Flow\Import\ImportException;
use MWTimestamp;

class MovedImportRevision extends ImportRevision {
	/**
	 * Rewrites the '#REDIRECT [[...]]' of an autogenerated lqt moved
	 * thread stub into a template.  While we don't re-write the link
	 * here, after importing the referenced thread LqtRedirector will
	 * make that Thread page a redirect to the Flow topic, essentially
	 * making these links still work.
	 * @return string
	 * @throws ImportException
	 */
	public function getText() {
		$text = parent::getText();
		$content = \ContentHandler::makeContent( $text, null, CONTENT_MODEL_WIKITEXT );
		$target = $content->getRedirectTarget();
		if ( !$target ) {
			throw new ImportException( "Could not detect redirect within: $text" );
		}

		// To get the new talk page that this belongs to we would need to query the api
		// for the new topic, for now not bothering.
		$template = wfMessage( 'flow-importer-lqt-moved-thread-template' )->inContentLanguage(
		)->plain();
		$arguments = implode(
			'|',
			[
				'author=' . parent::getAuthor(),
				'date=' . MWTimestamp::getInstance(
					$this->apiResponse['timestamp']
				)->timestamp->format( 'Y-m-d' ),
				'title=' . $target->getPrefixedText(),
			]
		);

		return "{{{$template}|$arguments}}";
	}
}
